@@ -7,18 +7,18 @@
 //! [climate orbiter][orbiter]!
 //!
 //! [analysis]: https://en.wikipedia.org/wiki/Dimensional_analysis
-//! [si]: http://jcgm.bipm.org/vim/en/1.16.html
-//! [isq]: http://jcgm.bipm.org/vim/en/1.6.html
-//! [quantity]: http://jcgm.bipm.org/vim/en/1.1.html
-//! [measurement]: http://jcgm.bipm.org/vim/en/1.9.html
+//! [si]: https://jcgm.bipm.org/vim/en/1.16.html
+//! [isq]: https://jcgm.bipm.org/vim/en/1.6.html
+//! [quantity]: https://jcgm.bipm.org/vim/en/1.1.html
+//! [measurement]: https://jcgm.bipm.org/vim/en/1.9.html
 //! [orbiter]: https://en.wikipedia.org/wiki/Mars_Climate_Orbiter
 //!
 //! ## Usage
-//! `uom` requires `rustc` 1.37.0 or later. Add this to your `Cargo.toml`:
+//! `uom` requires `rustc` 1.43.0 or later. Add this to your `Cargo.toml`:
 //!
 //! ```toml
 //! [dependencies]
-//! uom = "0.30.0"
+//! uom = "0.32.0"
 //! ```
 //!
 //! and this to your crate root:
@@ -66,7 +66,7 @@
 //! ```toml
 //! [dependencies]
 //! uom = {
-//!     version = "0.30.0",
+//!     version = "0.32.0",
 //!     default-features = false,
 //!     features = [
 //!         "autoconvert", # automatic base unit conversion.
@@ -76,7 +76,6 @@
 //!         "rational", "rational32", "rational64", "bigrational", # Integer ratio storage types.
 //!         "f32", "f64", # Floating point storage types.
 //!         "si", "std", # Built-in SI system and std library support.
-//!         "try-from", # `TryFrom` support between `Time` and `Duration`. Requires `rustc` 1.34.0.
 //!         "use_serde", # Serde support.
 //!     ]
 //! }
@@ -96,23 +95,21 @@
 //!    default.
 //!  * `std` -- Feature to compile with standard library support. Disabling this feature compiles
 //!    `uom` with `no_std`. Enabled by default.
-//!  * `try-from` -- Feature to enable `TryFrom` support between `Time` and `Duration`. Requires
-//!    `rustc` 1.34.0.
 //!  * `use_serde` -- Feature to enable support for serialization and deserialization of quantities
 //!    with the [Serde][serde] crate. Disabled by default.
 //!
-//! [si]: http://jcgm.bipm.org/vim/en/1.16.html
+//! [si]: https://jcgm.bipm.org/vim/en/1.16.html
 //! [serde]: https://serde.rs/
 //!
 //! ## Design
-//! Rather than working with [measurement units](http://jcgm.bipm.org/vim/en/1.9.html) (meter,
-//! kilometer, foot, mile, ...) `uom` works with [quantities](http://jcgm.bipm.org/vim/en/1.1.html)
+//! Rather than working with [measurement units](https://jcgm.bipm.org/vim/en/1.9.html) (meter,
+//! kilometer, foot, mile, ...) `uom` works with [quantities](https://jcgm.bipm.org/vim/en/1.1.html)
 //! (length, mass, time, ...). This simplifies usage because units are only involved at interface
 //! boundaries: the rest of your code only needs to be concerned about the quantities involved.
 //! This also makes operations on quantities (+, -, \*, /, ...) have zero runtime cost over using
 //! the raw storage type (e.g. `f32`).
 //!
-//! `uom` normalizes values to the [base unit](http://jcgm.bipm.org/vim/en/1.10.html) for the
+//! `uom` normalizes values to the [base unit](https://jcgm.bipm.org/vim/en/1.10.html) for the
 //! quantity. Alternative base units can be used by executing the macro defined for the system of
 //! quantities (`ISQ!` for the SI). `uom` supports `usize`, `u8`, `u16`, `u32`, `u64`, `u128`,
 //! `isize`, `i8`, `i16`, `i32`, `i64`, `i128`, `bigint`, `biguint`, `rational`, `rational32`,
@@ -141,14 +138,14 @@
 //! Licensed under either of
 //!
 //!  * Apache License, Version 2.0, ([LICENSE-APACHE](LICENSE-APACHE) or
-//!    <http://www.apache.org/licenses/LICENSE-2.0>)
-//!  * MIT license ([LICENSE-MIT](LICENSE-MIT) or <http://opensource.org/licenses/MIT>)
+//!    <https://www.apache.org/licenses/LICENSE-2.0>)
+//!  * MIT license ([LICENSE-MIT](LICENSE-MIT) or <https://opensource.org/licenses/MIT>)
 //!
 //! at your option.
 //!
-//! [BIPM]: http://www.bipm.org/en/about-us/
-//! [brochure]: http://www.bipm.org/en/publications/si-brochure/
-//! [si]: http://jcgm.bipm.org/vim/en/1.16.html
+//! [BIPM]: https://www.bipm.org/en/about-us/
+//! [brochure]: https://www.bipm.org/en/publications/si-brochure/
+//! [si]: https://jcgm.bipm.org/vim/en/1.16.html
 //! [nist811]: https://www.nist.gov/pml/nist-guide-si-appendix-b9-factors-units-listed-kind-quantity-or-field-science
 
 // Compile with `no_std` when the `std` feature is not specified.
@@ -221,8 +218,6 @@ extern crate approx;
 #[cfg(test)]
 #[macro_use]
 extern crate quickcheck;
-#[cfg(all(test, feature = "serde"))]
-extern crate serde_json;
 #[cfg(test)]
 #[macro_use]
 extern crate static_assertions;
@@ -246,6 +241,15 @@ pub mod lib {
         pub use std::ops::*;
         pub use typenum::type_operators::*;
     }
+
+    // Export `panic` module when the `std` feature is not enabled. `RefUnwindSafe` and `UnwindSafe`
+    // traits do not exist in `core` but are conditionally needed in traits defined by `uom` when
+    // `std` is enabled. These definitions work around conditional requirements.
+    #[cfg(not(feature = "std"))]
+    pub mod panic {
+        pub trait RefUnwindSafe {}
+        pub trait UnwindSafe {}
+    }
 }
 
 // Conditionally import num sub-crate types based on feature selection.
@@ -262,7 +266,7 @@ pub mod num {
     pub use num_bigint::{BigInt, BigUint};
 
     #[cfg(feature = "rational-support")]
-    pub use num_rational::Rational;
+    pub type Rational = num_rational::Ratio<isize>;
 
     #[cfg(feature = "bigint-support")]
     pub use num_rational::BigRational;
@@ -347,6 +351,9 @@ mod system;
 #[macro_use]
 mod quantity;
 
+#[macro_use]
+mod unit;
+
 #[cfg(feature = "si")]
 #[macro_use]
 pub mod si;
@@ -385,7 +392,7 @@ pub enum ConstantOp {
 /// ## Generic Parameters
 /// * `V`: Underlying storage type trait is implemented for.
 ///
-/// [units]: http://jcgm.bipm.org/vim/en/1.13.html
+/// [units]: https://jcgm.bipm.org/vim/en/1.13.html
 /// [factor]: https://jcgm.bipm.org/vim/en/1.24.html
 pub trait Conversion<V> {
     /// Conversion factor type specific to the underlying storage type.
@@ -415,7 +422,7 @@ pub trait Conversion<V> {
     ///
     /// Default implementation returns the coefficient: `Self::coefficient()`.
     #[inline(always)]
-    fn into_conversion(&self) -> Self::T
+    fn conversion(&self) -> Self::T
     where
         Self: Sized,
     {
@@ -429,8 +436,10 @@ pub trait Conversion<V> {
 /// * `V`: Underlying storage type trait is implemented for.
 ///
 /// [factor]: https://jcgm.bipm.org/vim/en/1.24.html
+#[allow(unused_qualifications)] // lib:cmp::PartialOrder false positive.
 pub trait ConversionFactor<V>:
-    lib::ops::Add<Self, Output = Self>
+    lib::cmp::PartialOrd
+    + lib::ops::Add<Self, Output = Self>
     + lib::ops::Sub<Self, Output = Self>
     + lib::ops::Mul<Self, Output = Self>
     + lib::ops::Div<Self, Output = Self>
@@ -444,6 +453,20 @@ pub trait ConversionFactor<V>:
     fn value(self) -> V;
 }
 
+/// Helper trait to identify the zero value of a type at compile time.
+///
+#[cfg_attr(all(feature = "si", feature = "f32"), doc = " ```rust")]
+#[cfg_attr(not(all(feature = "si", feature = "f32")), doc = " ```rust,ignore")]
+/// # use uom::si::f32::Length;
+/// use uom::ConstZero;
+///
+/// const ORIGIN: (Length, Length, Length) = (Length::ZERO, Length::ZERO, Length::ZERO);
+/// ```
+pub trait ConstZero {
+    /// Constant representing the zero value.
+    const ZERO: Self;
+}
+
 /// Default [kind][kind] of quantities to allow addition, subtraction, multiplication, division,
 /// remainder, negation, and saturating addition/subtraction.
 ///
@@ -479,7 +502,7 @@ storage_types! {
         }
 
         #[inline(always)]
-        fn into_conversion(&self) -> Self::T {
+        fn conversion(&self) -> Self::T {
             *self
         }
     }
@@ -495,6 +518,10 @@ storage_types! {
             self
         }
     }
+
+    impl crate::ConstZero for V {
+        const ZERO: Self = 0.0;
+    }
 }
 
 storage_types! {
@@ -504,7 +531,7 @@ storage_types! {
         type T = crate::num::rational::Ratio<V>;
 
         #[inline(always)]
-        fn into_conversion(&self) -> Self::T {
+        fn conversion(&self) -> Self::T {
             (*self).into()
         }
     }
@@ -520,6 +547,10 @@ storage_types! {
             self.to_integer()
         }
     }
+
+    impl crate::ConstZero for V {
+        const ZERO: Self = 0;
+    }
 }
 
 storage_types! {
@@ -529,7 +560,7 @@ storage_types! {
         type T = crate::num::rational::Ratio<V>;
 
         #[inline(always)]
-        fn into_conversion(&self) -> Self::T {
+        fn conversion(&self) -> Self::T {
             self.clone().into()
         }
     }
@@ -558,7 +589,7 @@ storage_types! {
         type T = V;
 
         #[inline(always)]
-        fn into_conversion(&self) -> Self::T {
+        fn conversion(&self) -> Self::T {
             *self
         }
     }
@@ -583,7 +614,7 @@ storage_types! {
         type T = V;
 
         #[inline(always)]
-        fn into_conversion(&self) -> Self::T {
+        fn conversion(&self) -> Self::T {
             self.clone()
         }
     }
@@ -621,6 +652,8 @@ pub mod fmt {
 
 /// Unicode string slice manipulation for quantities.
 pub mod str {
+    use crate::lib::fmt::{self, Display, Formatter};
+
     /// Represents an error encountered while parsing a string into a `Quantity`.
     #[allow(missing_copy_implementations)]
     #[derive(Clone, Debug, Eq, PartialEq)]
@@ -641,4 +674,19 @@ pub mod str {
         /// unit name (description) is correct.
         UnknownUnit,
     }
+
+    impl Display for ParseQuantityError {
+        fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
+            use ParseQuantityError::*;
+
+            match *self {
+                NoSeparator => write!(f, "no space between quantity and units"),
+                ValueParseError => write!(f, "error parsing unit quantity"),
+                UnknownUnit => write!(f, "unrecognized unit of measure"),
+            }
+        }
+    }
+
+    #[cfg(feature = "std")]
+    impl crate::lib::error::Error for ParseQuantityError {}
 }
