@@ -1,4 +1,4 @@
-/// Macro to implement a [system of quantities](http://jcgm.bipm.org/vim/en/1.3.html). `@...` match
+/// Macro to implement a [system of quantities](https://jcgm.bipm.org/vim/en/1.3.html). `@...` match
 /// arms are considered private.
 ///
 /// * `$quantities_attr`: System of quantities attributes. Generally used to set documentation
@@ -119,12 +119,18 @@ macro_rules! system {
         /// [base quantities][base] of a [system of quantities][quantities] as a product of powers
         /// of factors corresponding to the base quantities, omitting any numerical factor.
         ///
-        /// * <http://jcgm.bipm.org/vim/en/1.7.html>
+        /// * <https://jcgm.bipm.org/vim/en/1.7.html>
         ///
-        /// [quantity]: http://jcgm.bipm.org/vim/en/1.1.html
-        /// [base]: http://jcgm.bipm.org/vim/en/1.4.html
-        /// [quantities]: http://jcgm.bipm.org/vim/en/1.3.html
-        pub trait Dimension: Send + Sync {
+        /// [quantity]: https://jcgm.bipm.org/vim/en/1.1.html
+        /// [base]: https://jcgm.bipm.org/vim/en/1.4.html
+        /// [quantities]: https://jcgm.bipm.org/vim/en/1.3.html
+        pub trait Dimension:
+            Send
+            + Sync
+            + Unpin
+            + $crate::lib::panic::RefUnwindSafe
+            + $crate::lib::panic::UnwindSafe
+        {
             $($(#[$name_attr])*
             ///
             /// Quantity dimension.
@@ -141,10 +147,15 @@ macro_rules! system {
         /// ## Generic Parameters
         /// * `V`: Underlying storage type trait is implemented for.
         ///
-        /// [units]: http://jcgm.bipm.org/vim/en/1.13.html
-        /// [base]: http://jcgm.bipm.org/vim/en/1.10.html
-        /// [quantities]: http://jcgm.bipm.org/vim/en/1.3.html
-        pub trait Units<V>: Send + Sync
+        /// [units]: https://jcgm.bipm.org/vim/en/1.13.html
+        /// [base]: https://jcgm.bipm.org/vim/en/1.10.html
+        /// [quantities]: https://jcgm.bipm.org/vim/en/1.3.html
+        pub trait Units<V>:
+            Send
+            + Sync
+            + Unpin
+            + $crate::lib::panic::RefUnwindSafe
+            + $crate::lib::panic::UnwindSafe
         where
             V: $crate::Conversion<V>,
         {
@@ -158,8 +169,8 @@ macro_rules! system {
         /// Trait to identify [measurement units][measurement] of individual
         /// [quantities][quantity].
         ///
-        /// [measurement]: http://jcgm.bipm.org/vim/en/1.9.html
-        /// [quantity]: http://jcgm.bipm.org/vim/en/1.1.html
+        /// [measurement]: https://jcgm.bipm.org/vim/en/1.9.html
+        /// [quantity]: https://jcgm.bipm.org/vim/en/1.1.html
         pub trait Unit: Copy {
             /// Unit abbreviation.
             fn abbreviation() -> &'static str;
@@ -234,7 +245,7 @@ macro_rules! system {
         /// let v: Velocity = Length::new::<meter>(1.0) * Time::new::<second>(1.0);
         /// ```
         ///
-        /// * <http://jcgm.bipm.org/vim/en/1.1.html>
+        /// * <https://jcgm.bipm.org/vim/en/1.1.html>
         ///
         /// ## Generic Parameters
         /// * `D`: Quantity dimension. See [`Dimension`](./trait.Dimension.html).
@@ -264,8 +275,8 @@ macro_rules! system {
         /// Type alias for [dimension one][one] for which all the exponents of the factors
         /// corresponding to the [base quantities][base] are zero.
         ///
-        /// [one]: http://jcgm.bipm.org/vim/en/1.8.html
-        /// [base]: http://jcgm.bipm.org/vim/en/1.4.html
+        /// [one]: https://jcgm.bipm.org/vim/en/1.8.html
+        /// [base]: https://jcgm.bipm.org/vim/en/1.4.html
         #[allow(dead_code)]
         pub type DimensionOne = DN<$crate::typenum::Z0>;
 
@@ -292,16 +303,23 @@ macro_rules! system {
         where
             D: Dimension + ?Sized,
             U: Units<V> + ?Sized,
-            V: $crate::Conversion<V> + $crate::lib::ops::Mul<V, Output = V>,
+            V: $crate::Conversion<V>,
             N: $crate::Conversion<V, T = V::T>,
         {
             use $crate::typenum::Integer;
-            use $crate::Conversion;
-            use $crate::ConversionFactor;
+            use $crate::{Conversion, ConversionFactor};
 
-            (v.into_conversion() $(* U::$name::coefficient().powi(D::$symbol::to_i32()))+
-                    / N::coefficient() - N::constant($crate::ConstantOp::Sub))
-                .value()
+            let v = v.conversion();
+            let n_coef = N::coefficient();
+            let f = V::coefficient() $(* U::$name::coefficient().powi(D::$symbol::to_i32()))+;
+            let n_cons = N::constant($crate::ConstantOp::Sub);
+
+            if n_coef < f {
+                (v * (f / n_coef) - n_cons).value()
+            }
+            else {
+                (v / (n_coef / f) - n_cons).value()
+            }
         }
 
         /// Convert a value from the given unit to base units.
@@ -316,16 +334,23 @@ macro_rules! system {
         where
             D: Dimension + ?Sized,
             U: Units<V> + ?Sized,
-            V: $crate::Conversion<V> + $crate::lib::ops::Mul<V, Output = V>,
+            V: $crate::Conversion<V>,
             N: $crate::Conversion<V, T = V::T>,
         {
             use $crate::typenum::Integer;
-            use $crate::Conversion;
-            use $crate::ConversionFactor;
+            use $crate::{Conversion, ConversionFactor};
 
-            ((v.into_conversion() + N::constant($crate::ConstantOp::Add)) * N::coefficient()
-                    / (V::coefficient() $(* U::$name::coefficient().powi(D::$symbol::to_i32()))+))
-                .value()
+            let v = v.conversion();
+            let n_coef = N::coefficient();
+            let f = V::coefficient() $(* U::$name::coefficient().powi(D::$symbol::to_i32()))+;
+            let n_cons = N::constant($crate::ConstantOp::Add);
+
+            if n_coef >= f {
+                ((v + n_cons) * (n_coef / f)).value()
+            }
+            else {
+                (((v + n_cons) * n_coef) / f).value()
+            }
         }
 
         autoconvert_test! {
@@ -346,10 +371,9 @@ macro_rules! system {
             V: $crate::Conversion<V> + $crate::lib::ops::Mul<V, Output = V>,
         {
             use $crate::typenum::Integer;
-            use $crate::Conversion;
-            use $crate::ConversionFactor;
+            use $crate::{Conversion, ConversionFactor};
 
-            (v.into_conversion() $(* Ur::$name::coefficient().powi(D::$symbol::to_i32())
+            (v.conversion() $(* Ur::$name::coefficient().powi(D::$symbol::to_i32())
                     / Ul::$name::coefficient().powi(D::$symbol::to_i32()))+)
                 .value()
         }}
@@ -471,7 +495,8 @@ macro_rules! system {
                     for Quantity<Dl, U, V>
                 where
                     Dl: Dimension + ?Sized,
-                    $(Dl::$symbol: $crate::lib::ops::$AddSubTrait<Dr::$symbol>,)+
+                    $(Dl::$symbol: $crate::lib::ops::$AddSubTrait<Dr::$symbol>,
+                    <Dl::$symbol as $crate::lib::ops::$AddSubTrait<Dr::$symbol>>::Output: $crate::typenum::Integer,)+
                     Dl::Kind: $crate::marker::$MulDivTrait,
                     Dr: Dimension + ?Sized,
                     Dr::Kind: $crate::marker::$MulDivTrait,
@@ -809,7 +834,9 @@ macro_rules! system {
             #[cfg_attr(not(all(feature = "si", feature = "f32")), doc = " ```rust,ignore")]
             /// # use uom::si::f32::*;
             /// # use uom::si::length::meter;
-            /// let a: Area = Length::new::<meter>(3.0).powi(crate::uom::typenum::P2::new());
+            /// use uom::typenum::P2;
+            ///
+            /// let a: Area = Length::new::<meter>(3.0).powi(P2::new());
             /// ```
             ///
             /// ## Generic Parameters
@@ -1030,17 +1057,6 @@ macro_rules! system {
             }
         }
 
-        // Implement Unpin manually because the auto-impl only kicks in when D,
-        // U, and V are Unpin. However because D and U don't exist at runtime,
-        // only V is required to be Unpin.
-        impl<D, U, V> $crate::lib::marker::Unpin for Quantity<D, U, V>
-        where
-            D: Dimension + ?Sized,
-            U: Units<V> + ?Sized,
-            V: $crate::num::Num + $crate::Conversion<V> + $crate::lib::marker::Unpin,
-        {
-        }
-
         autoconvert! {
         impl<D, Ul, Ur, V> $crate::lib::cmp::PartialEq<Quantity<D, Ur, V>> for Quantity<D, Ul, V>
         where
@@ -1287,6 +1303,19 @@ macro_rules! system {
             }
         }
 
+        impl<D, U, V> $crate::ConstZero for Quantity<D, U, V>
+        where
+            D: Dimension + ?Sized,
+            U: Units<V> + ?Sized,
+            V: $crate::num::Num + $crate::Conversion<V> + $crate::ConstZero,
+        {
+            const ZERO: Self = Self {
+                dimension: $crate::lib::marker::PhantomData,
+                units: $crate::lib::marker::PhantomData,
+                value: V::ZERO,
+            };
+        }
+
         serde! {
         impl<D, U, V> $crate::serde::Serialize for Quantity<D, U, V>
         where
@@ -1424,13 +1453,6 @@ macro_rules! system {
             {
             }
 
-            impl<D, N> $crate::lib::marker::Unpin for Arguments<D, N>
-            where
-                D: Dimension + ?Sized,
-                N: Unit + $crate::lib::marker::Unpin,
-            {
-            }
-
             impl<D, U, V, N> $crate::lib::clone::Clone for QuantityArguments<D, U, V, N>
             where
                 D: Dimension + ?Sized,
@@ -1455,15 +1477,6 @@ macro_rules! system {
             {
             }
 
-            impl<D, U, V, N> $crate::lib::marker::Unpin for QuantityArguments<D, U, V, N>
-            where
-                D: Dimension + ?Sized,
-                U: Units<V> + ?Sized,
-                V: $crate::num::Num + $crate::Conversion<V> + $crate::lib::marker::Unpin,
-                N: Unit + $crate::lib::marker::Unpin,
-            {
-            }
-
             macro_rules! format_arguments {
                 ($style:ident) => {
                     impl<D, U, V, N> fmt::$style for QuantityArguments<D, U, V, N>
@@ -1502,8 +1515,8 @@ macro_rules! system {
         /// Macro to implement [`quantity`](si/struct.Quantity.html) type aliases for a specific
         /// [system of units][units] and value storage type.
         ///
-        /// * `$path`: Path to the module where the [`system!`](macro.system.html) macro was run
-        ///   (e.g. `::uom::si`).
+        /// * `$system`: Path to the module where the [`system!`](macro.system.html) macro was run
+        ///   (e.g. `uom::si`).
         /// * `$V`: Underlying value storage type (e.g. `f32`).
         /// * `$U`: Optional. Base units. Pass as a tuple with the desired units: `(meter, kilogram,
         ///   second, ampere, kelvin, mole, candela)`. The system's base units will be used if no
@@ -1578,7 +1591,7 @@ macro_rules! system {
         /// # }
         /// ```
         ///
-        /// [units]: http://jcgm.bipm.org/vim/en/1.13.html
+        /// [units]: https://jcgm.bipm.org/vim/en/1.13.html
         #[macro_export]
         macro_rules! $quantities {
             ($path:path) => {
