@@ -38,6 +38,8 @@ storage_types! {
         let m2 = k::Mass::new::<kilogram>(V::from_f64(1.0E-3).unwrap());
 
         Test::assert_eq(&"1 m".parse::<k::Length>().unwrap(), &l1);
+        Test::assert_eq(&"1 meter".parse::<k::Length>().unwrap(), &l1);
+        Test::assert_eq(&"1 meters".parse::<k::Length>().unwrap(), &l1);
         Test::assert_eq(&"1.0 km".parse::<k::Length>().unwrap(), &l2);
         Test::assert_eq(&"1000 kg".parse::<k::Mass>().unwrap(), &m1);
         Test::assert_eq(&"1.0E-3 kg".parse::<k::Mass>().unwrap(), &m2);
@@ -52,30 +54,55 @@ storage_types! {
     #[cfg(feature = "autoconvert")]
     quickcheck! {
         #[allow(trivial_casts)]
-        fn add(l: A<V>, r: A<V>) -> bool {
-            Test::approx_eq(&k::Length::new::<meter>(&*l + &*r),
-                &(k::Length::new::<meter>((*l).clone())
-                    + f::Length::new::<meter>((*r).clone())))
+        fn add(l: A<V>, r: A<V>) -> TestResult {
+            let km: V = <kilometer as crate::Conversion<V>>::coefficient().value();
+            let f = (&*l + &*r) / &km;
+            let i = (&*l / &km) + (&*r / &km);
+
+            if !Test::approx_eq(&i, &f) {
+                return TestResult::discard();
+            }
+
+            TestResult::from_bool(
+                Test::approx_eq(&k::Length::new::<meter>(&*l + &*r),
+                    &(k::Length::new::<meter>((*l).clone())
+                        + f::Length::new::<meter>((*r).clone()))))
         }
 
         #[allow(trivial_casts)]
-        fn sub(l: A<V>, r: A<V>) -> bool {
-            Test::approx_eq(&k::Length::new::<meter>(&*l - &*r),
-                &(k::Length::new::<meter>((*l).clone())
-                    - f::Length::new::<meter>((*r).clone())))
+        fn sub(l: A<V>, r: A<V>) -> TestResult {
+            let km: V = <kilometer as crate::Conversion<V>>::coefficient().value();
+            let f = (&*l - &*r) / &km;
+            let i = (&*l / &km) - (&*r / &km);
+
+            if !Test::approx_eq(&i, &f) {
+                return TestResult::discard();
+            }
+
+            TestResult::from_bool(
+                Test::approx_eq(&k::Length::new::<meter>(&*l - &*r),
+                    &(k::Length::new::<meter>((*l).clone())
+                        - f::Length::new::<meter>((*r).clone()))))
         }
 
         #[allow(trivial_casts)]
-        fn mul_quantity(l: A<V>, r: A<V>) -> bool {
-            Test::approx_eq(&/*Area::new::<square_meter>*/(&*l * &*r),
-                    &(f::Length::new::<meter>((*l).clone())
-                        * k::Length::new::<meter>((*r).clone())).value)
-                && Test::approx_eq(&/*Area::new::<square_meter>*/(&*l * &*r),
-                    &(f::Length::new::<meter>((*l).clone())
-                        * k::Mass::new::<kilogram>((*r).clone())).value)
-                && Test::approx_eq(&/*Area::new::<square_kilometer>*/(&*l * &*r),
-                    &(k::Length::new::<kilometer>((*l).clone())
-                        * f::Mass::new::<kilogram>((*r).clone())).value)
+        fn mul_quantity(l: A<V>, r: A<V>) -> TestResult {
+            let km: V = <kilometer as crate::Conversion<V>>::coefficient().value();
+
+            TestResult::from_bool(
+                Test::approx_eq(&/*Area::new::<square_meter>*/(&*l * &*r),
+                        &(f::Length::new::<meter>((*l).clone())
+                            * k::Length::new::<meter>((*r).clone())).value)
+                    && Test::approx_eq(
+                        &/*Area::new::<square_kilometer>*/(&(&*l / &km) * &(&*r / &km)),
+                        &(k::Length::new::<meter>((*l).clone())
+                            * f::Length::new::<meter>((*r).clone())).value)
+                    && Test::approx_eq(&/*Length-mass*/(&*l * &*r),
+                        &(f::Length::new::<meter>((*l).clone())
+                            * k::Mass::new::<kilogram>((*r).clone())).value)
+                    && Test::approx_eq(&/*Length-mass*/(&*l * &*r),
+                        &(k::Length::new::<kilometer>((*l).clone())
+                            * f::Mass::new::<kilogram>((*r).clone())).value))
         }
 
         #[allow(trivial_casts)]
@@ -84,11 +111,21 @@ storage_types! {
                 return TestResult::discard();
             }
 
-            // TODO Use `.get(?)`, add ratio type?
+            let km: V = <kilometer as crate::Conversion<V>>::coefficient().value();
+
             TestResult::from_bool(
-                Test::approx_eq(&(&*l / &*r),
-                    &(k::Length::new::<meter>((*l).clone())
-                        / f::Length::new::<meter>((*r).clone())).value))
+                Test::approx_eq(&/*Ratio::new::<ratio>*/(&*l / &*r),
+                        &(f::Length::new::<meter>((*l).clone())
+                            / k::Length::new::<meter>((*r).clone())).value)
+                    && Test::approx_eq(&/*Ratio::new::<ratio>*/(&*l / &*r),
+                        &(k::Length::new::<meter>((*l).clone())
+                            / f::Length::new::<meter>((*r).clone())).value)
+                    && Test::approx_eq(&/*Length/mass*/(&*l / &*r),
+                        &(f::Length::new::<meter>((*l).clone())
+                            / k::Mass::new::<kilogram>((*r).clone())).value)
+                    && Test::approx_eq(&/*Length/mass*/(&*l / &km / &*r),
+                        &(k::Length::new::<meter>((*l).clone())
+                            / f::Mass::new::<kilogram>((*r).clone())).value))
         }
 
         #[allow(trivial_casts)]
@@ -97,6 +134,14 @@ storage_types! {
                 return TestResult::discard();
             }
 
+            let km: V = <kilometer as crate::Conversion<V>>::coefficient().value();
+            let f = (&*l % &*r) / &km;
+            let i = (&*l / &km) % (&*r / &km);
+
+            if !Test::approx_eq(&i, &f) {
+                return TestResult::discard();
+            }
+
             TestResult::from_bool(
                 Test::approx_eq(&k::Length::new::<meter>(&*l % &*r),
                     &(k::Length::new::<meter>((*l).clone())
@@ -105,79 +150,93 @@ storage_types! {
 
         #[allow(trivial_casts)]
         fn eq(l: A<V>, r: A<V>) -> bool {
-            let a = *l == *r;
-            let b = f::Length::new::<meter>((*l).clone())
+            let km: V = <kilometer as crate::Conversion<V>>::coefficient().value();
+            let a = *l == ((*r).clone() / &km) * &km;
+            let b = (*l).clone() / &km == (*r).clone() / &km;
+            let x = f::Length::new::<meter>((*l).clone())
                 == k::Length::new::<meter>((*r).clone());
-            let c = k::Length::new::<meter>((*l).clone())
+            let y = k::Length::new::<meter>((*l).clone())
                 == f::Length::new::<meter>((*r).clone());
 
-            a == b && a == c
+            a == x && b == y
         }
 
         #[allow(trivial_casts)]
         fn ne(l: A<V>, r: A<V>) -> bool {
-            let a = *l != *r;
-            let b = f::Length::new::<meter>((*l).clone())
+            let km: V = <kilometer as crate::Conversion<V>>::coefficient().value();
+            let a = *l != ((*r).clone() / &km) * &km;
+            let b = (*l).clone() / &km != (*r).clone() / &km;
+            let x = f::Length::new::<meter>((*l).clone())
                 != k::Length::new::<meter>((*r).clone());
-            let c = k::Length::new::<meter>((*l).clone())
+            let y = k::Length::new::<meter>((*l).clone())
                 != f::Length::new::<meter>((*r).clone());
 
-            a == b && a == c
+            a == x && b == y
         }
 
         #[allow(trivial_casts)]
         fn partial_cmp(l: A<V>, r: A<V>) -> bool {
-            let a = (*l).partial_cmp(&*r);
-            let b = f::Length::new::<meter>((*l).clone()).partial_cmp(
+            let km: V = <kilometer as crate::Conversion<V>>::coefficient().value();
+            let a = (*l).partial_cmp(&(((*r).clone() / &km) * &km));
+            let b = ((*l).clone() / &km).partial_cmp(&((*r).clone() / &km));
+            let x = f::Length::new::<meter>((*l).clone()).partial_cmp(
                 &k::Length::new::<meter>((*r).clone()));
-            let c = k::Length::new::<meter>((*l).clone()).partial_cmp(
+            let y = k::Length::new::<meter>((*l).clone()).partial_cmp(
                 &f::Length::new::<meter>((*r).clone()));
 
-            a == b && a == c
+            a == x && b == y
         }
 
         #[allow(trivial_casts)]
         fn lt(l: A<V>, r: A<V>) -> bool {
-            let a = (*l).lt(&*r);
-            let b = f::Length::new::<meter>((*l).clone()).lt(
+            let km: V = <kilometer as crate::Conversion<V>>::coefficient().value();
+            let a = (*l).lt(&(((*r).clone() / &km) * &km));
+            let b = ((*l).clone() / &km).lt(&((*r).clone() / &km));
+            let x = f::Length::new::<meter>((*l).clone()).lt(
                 &k::Length::new::<meter>((*r).clone()));
-            let c = k::Length::new::<meter>((*l).clone()).lt(
+            let y = k::Length::new::<meter>((*l).clone()).lt(
                 &f::Length::new::<meter>((*r).clone()));
 
-            a == b && a == c
+            a == x && b == y
         }
 
         #[allow(trivial_casts)]
         fn le(l: A<V>, r: A<V>) -> bool {
-            let a = (*l).le(&*r);
-            let b = f::Length::new::<meter>((*l).clone()).le(
+            let km: V = <kilometer as crate::Conversion<V>>::coefficient().value();
+            let a = (*l).le(&(((*r).clone() / &km) * &km));
+            let b = ((*l).clone() / &km).le(&((*r).clone() / &km));
+            let x = f::Length::new::<meter>((*l).clone()).le(
                 &k::Length::new::<meter>((*r).clone()));
-            let c = k::Length::new::<meter>((*l).clone()).le(
+            let y = k::Length::new::<meter>((*l).clone()).le(
                 &f::Length::new::<meter>((*r).clone()));
 
-            a == b && a == c
+            a == x && b == y
         }
 
         #[allow(trivial_casts)]
         fn gt(l: A<V>, r: A<V>) -> bool {
-            let a = (*l).gt(&*r);
-            let b = f::Length::new::<meter>((*l).clone()).gt(
+            let km: V = <kilometer as crate::Conversion<V>>::coefficient().value();
+            let a = (*l).gt(&(((*r).clone() / &km) * &km));
+            let b = ((*l).clone() / &km).gt(&((*r).clone() / &km));
+            let x = f::Length::new::<meter>((*l).clone()).gt(
                 &k::Length::new::<meter>((*r).clone()));
-            let c = k::Length::new::<meter>((*l).clone()).gt(
+            let y = k::Length::new::<meter>((*l).clone()).gt(
                 &f::Length::new::<meter>((*r).clone()));
 
-            a == b && a == c
+            a == x && b == y
         }
 
         #[allow(trivial_casts)]
         fn ge(l: A<V>, r: A<V>) -> bool {
-            let a = (*l).ge(&*r);
-            let b = f::Length::new::<meter>((*l).clone()).ge(
+            let km: V = <kilometer as crate::Conversion<V>>::coefficient().value();
+            let a = (*l).ge(&(((*r).clone() / &km) * &km));
+            let b = ((*l).clone() / &km).ge(&((*r).clone() / &km));
+            let x = f::Length::new::<meter>((*l).clone()).ge(
                 &k::Length::new::<meter>((*r).clone()));
-            let c = k::Length::new::<meter>((*l).clone()).ge(
+            let y = k::Length::new::<meter>((*l).clone()).ge(
                 &f::Length::new::<meter>((*r).clone()));
 
-            a == b && a == c
+            a == x && b == y
         }
     }
 }
@@ -300,25 +359,39 @@ mod non_big {
 
         quickcheck! {
             #[allow(trivial_casts)]
-            fn add_assign(l: A<V>, r: A<V>) -> bool {
+            fn add_assign(l: A<V>, r: A<V>) -> TestResult {
+                let km: V = <kilometer as crate::Conversion<V>>::coefficient().value();
                 let mut f = *l;
+                let mut i = *l / &km;
                 let mut v = k::Length::new::<meter>(*l);
 
                 f += *r;
+                i += *r / &km;
                 v += f::Length::new::<meter>(*r);
 
-                Test::approx_eq(&k::Length::new::<meter>(f), &v)
+                if !Test::approx_eq(&i, &(f / &km)) {
+                    return TestResult::discard();
+                }
+
+                TestResult::from_bool(Test::approx_eq(&k::Length::new::<meter>(f), &v))
             }
 
             #[allow(trivial_casts)]
-            fn sub_assign(l: A<V>, r: A<V>) -> bool {
+            fn sub_assign(l: A<V>, r: A<V>) -> TestResult {
+                let km: V = <kilometer as crate::Conversion<V>>::coefficient().value();
                 let mut f = *l;
+                let mut i = *l / &km;
                 let mut v = k::Length::new::<meter>(*l);
 
                 f -= *r;
+                i -= *r / &km;
                 v -= f::Length::new::<meter>(*r);
 
-                Test::approx_eq(&k::Length::new::<meter>(f), &v)
+                if !Test::approx_eq(&i, &(f / &km)) {
+                    return TestResult::discard();
+                }
+
+                TestResult::from_bool(Test::approx_eq(&k::Length::new::<meter>(f), &v))
             }
 
             #[allow(trivial_casts)]
@@ -327,12 +400,19 @@ mod non_big {
                     return TestResult::discard();
                 }
 
+                let km: V = <kilometer as crate::Conversion<V>>::coefficient().value();
                 let mut f = *l;
+                let mut i = *l / &km;
                 let mut v = k::Length::new::<meter>(*l);
 
                 f %= *r;
+                i %= *r / &km;
                 v %= f::Length::new::<meter>(*r);
 
+                if !Test::approx_eq(&i, &(f / km)) {
+                    return TestResult::discard();
+                }
+
                 TestResult::from_bool(Test::approx_eq(&k::Length::new::<meter>(f), &v))
             }
         }
